// Copyright 2015 Open Source Robotics Foundation, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include <chrono>
#include <memory>
#include <string>
#include <thread>

#include "priority_executor/default_executor.hpp"
#include "priority_executor/multithread_priority_executor.hpp"
#include "rclcpp/rclcpp.hpp"
#include "std_msgs/msg/string.hpp"

using namespace std::chrono_literals;

void chatter_callback(std_msgs::msg::String::SharedPtr const msg) {
    RCLCPP_INFO(rclcpp::get_logger("rclcpp"), "I heard: '%s'", msg->data.c_str());
}

int main(int argc, char* argv[]) {
    rclcpp::init(argc, argv);

    auto node = rclcpp::Node::make_shared("minimal_subscriber");

    auto subscription = node->create_subscription<std_msgs::msg::String>(
        "topic", 10, chatter_callback);

    auto executor = std::make_shared<priority_executor::ROSDefaultExecutor>();
    executor->add_node(node);
    executor->spin();

    rclcpp::shutdown();
    return 0;
}
